# -*- coding: utf-8 -*-
"""fluid_design_system implementation of base components

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.registry import yes

from cubicweb_web.views.basecomponents import HeaderComponent
from cubicweb_web.views.basetemplates import TheMainTemplate

from cubicweb_web import component

from cubicweb_fluid_design_system import monkeypatch_default_value

monkeypatch_default_value(component.CtxComponent.render_items, "klass", "list-unstyled")


class HideAsidesBar(HeaderComponent):
    """Hide the left bar"""

    __regid__ = "hide-left-bar"
    __select__ = yes()
    context = "header-right"
    order = 3
    visible = False
    icon_css_cls = "glyphicon glyphicon-align-justify"

    def render(self, w):
        define_var = self._cw.html_headers.define_var
        define_var("twbs_col_cls", TheMainTemplate.twbs_col_cls)
        define_var("twbs_col_size", TheMainTemplate.twbs_col_size)
        define_var("twbs_grid_columns", TheMainTemplate.twbs_grid_columns)
        w(
            """<button class="btn btn-default navbar-btn" id="cw-aside-toggle"
              onclick="cw.cubes.squareui.toggleLeftColumn()" title="%s">
              <span class="%s"></span>
              </button>"""
            % (
                self._cw._("collapse boxes"),
                self.icon_css_cls,
            )
        )



"""
:copyright 2012 CreaLibre (Monterrey, MEXICO), all rights reserved.
:contact http://www.crealibre.com/ -- mailto:info@crealibre.com

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

# flake8: noqa


STYLESHEETS = [
    data("css/bootstrap.min.css"),
    data("css/fix.css"),
    data("cubes.bootstrap.css"),
    data("cubicweb.pictograms.css"),
    "https://cdn.jsdelivr.net/npm/@engie-group/fluid-design-system@v4.10.1/lib/fluid-design-system.css",
    "https://fonts.googleapis.com/css?family=Material+Icons|Lato:300,400,700,900&display=swap",
]

CW_COMPAT_STYLESHEETS = [data("cubes.bootstrap.cw_compat.css")]

JAVASCRIPTS.extend(
    (
        data("js/bootstrap.min.js"),
        data("cubes.bootstrap.js"),
        data("cubes.squareui.js"),
        "https://cdn.jsdelivr.net/npm/@engie-group/fluid-design-system@v4.10.1/lib/fluid-design-system.js",
        "https://cdn.jsdelivr.net/npm/@engie-group/fluid-design-system@v4.10.1/lib/auto-init.js",
    )
)



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of boxes

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb.utils import wrap_on_write
from cubicweb_web.views import boxes

boxes.SearchBox.formdef = (
    '<form action="%(action)s" id="search_box" class="navbar-form" role="search">\n'
    '  <input id="norql" type="text" accesskey="q" tabindex="%(tabindex1)s"'
    '    title="search text" value="%(value)s" name="rql"'
    '    class="search-query form-control" placeholder="%(searchlabel)s"/>\n'
    '  <input type="hidden" name="__fromsearchbox" value="1" />\n'
    '  <input type="hidden" name="subvid" value="tsearch" />\n'
    "</form>\n"
)


@monkeypatch(boxes.SearchBox)
def render_body(self, w):
    # Don't display search box title, just display the search box body
    if self._cw.form.pop("__fromsearchbox", None):
        rql = self._cw.form.get("rql", "")
    else:
        rql = ""
    w(
        self.formdef
        % {
            "action": self._cw.build_url("view"),
            "tabindex1": "",
            "value": xml_escape(rql),
            "searchlabel": self._cw._("Search"),
        }
    )


@monkeypatch(boxes.ContextualBoxLayout)
def render(self, w):
    if self.init_rendering():
        view = self.cw_extra_kwargs["view"]
        w(
            '<div class="panel panel-default %s %s" id="%s">'
            % (self.cssclass, view.cssclass, view.domid)
        )
        with wrap_on_write(
            w, '<div class="panel-heading"><div class="panel-title">', "</div></div>"
        ) as wow:
            view.render_title(wow)
        w('<div class="panel-body">')
        view.render_body(w)
        # We dissapear the boxFooter CSS place holder, as shadows
        # or effect will be made with CSS
        w("</div></div>\n")



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of base components

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb_web.component import Link
from cubicweb_web.views import basecomponents
from cubicweb.uilib import js

from cubicweb_fluid_design_system.views import utils


basecomponents.CookieLoginComponent._html = (
    '<a class="icon-login" data-toggle="modal" href="#loginModal">%s</a>'
)


@monkeypatch(basecomponents.CookieLoginComponent)
def call(self):
    self.w(self._html, self._cw._("i18n_login_popup"))
    self._cw.add_onload(
        js.jQuery("body").append(
            self._cw.view(
                "logform",
                rset=self.cw_rset,
                id=self.loginboxid,
                klass=self.loginboxid,
                title=True,
                showmessage=False,
                showonload=False,
            )
        )
    )
    self._cw.add_onload(
        """
       $('#loginModal').on('shown.bs.modal', function () {
           $('#__login:visible').focus();
       });"""
    )


# NOTE: CW 3.18 may introduce render_messages(). This would be the
#       the only method to override
@monkeypatch(basecomponents.ApplicationMessage)  # noqa: F811
def call(self, msg=None):
    if msg is None:
        msg = self._cw.message  # XXX don't call self._cw.message twice
    if msg:
        self.w(
            '<div class="alert alert-info" id="%s">'
            '<button class="close" data-dismiss="alert" type="button">x</button>'
            " %s</div>",
            self.domid,
            msg,
            escape=False,
        )


@monkeypatch(basecomponents.ApplLogo)
def render(self, w):
    w('<a id="logo" href="%s"></a>' % self._cw.base_url())


@monkeypatch(basecomponents.ApplicationName)  # noqa: F811
def render(self, w, **kwargs):
    title = self._cw.property_value("ui.site-title")
    if title:
        w(
            '<a class="cw-site-title" href="%s">%s</a>'
            % (self._cw.base_url(), xml_escape(title))
        )


class BSAuthenticatedUserStatus(basecomponents.AuthenticatedUserStatus):
    divider_html = '<li class="divider"></li>'

    def render(self, w):
        # display useractions and siteactions
        actions = self._cw.vreg["actions"].possible_actions(
            self._cw, rset=self.cw_rset, view=self.cw_extra_kwargs["view"]
        )
        html = []
        for action in actions.get("useractions", ()):
            self.render_actions(html.append, action)
        if actions.get("useractions") and actions.get("siteactions"):
            html.append(self.divider_html)
        for action in actions.get("siteactions", ()):
            self.render_actions(html.append, action)
        title = self.generate_initial_from_user(self._cw.user)
        ddb = AUSDropDownBox(title, "\n".join(html))
        ddb.render(w=w)

    def generate_initial_from_user(self, user):
        firstname = "" if user.firstname is None else user.firstname.strip()
        lastname = "" if user.surname is None else user.surname.strip()

        if firstname and lastname:
            return lastname[0].upper() + firstname[0].upper()
        elif firstname and not lastname:
            return firstname[0].upper()
        elif not firstname and lastname:
            return lastname[0].upper()

        return user.login[:2].upper()

    def render_actions(self, w, action):
        self.action_link(action).render(w=w)

    def action_link(self, action):
        kwargs = {}
        if action.title == "logout":
            kwargs["icon"] = "logout"
        return self.link(self._cw._(action.title), action.url(), **kwargs)

    def link(self, title, url, **kwargs):
        kwargs["klass"] = "nj-list-group__item"
        return FDSLink(url, title, **kwargs)


class FDSLink(Link):

    def render(self, w):
        label = self.label
        if "icon" in self.attrs:
            label = (
                f"<i class='material-icons mr-1'>{self.attrs['icon']}</i>{self.label}"
            )
        w(f"<a href='{self.href}' class='{self.attrs.get('klass', '')}'>{label}</a>")


class AUSDropDownBox(utils.DropDownBox):
    def render(self, w):
        if not len(self.actions):
            return ""
        w(
            self.ul_template
            % {"title": self.title, "actions": self.actions, "klass": self.klass}
        )


class BSRQLInputForm(basecomponents.RQLInputForm):
    """build the rql input form, usually displayed in the header"""

    __regid__ = "rqlinput"
    cw_property_defs = basecomponents.VISIBLE_PROP_DEF
    visible = False
    formdef = """<form id="rqlinput" action="%s" class="%s" role="search">
    <input type="text" id="rql" name="rql" value="%s" placeholder="%s" class="form-control"
       title="%s" tabindex="%s" accesskey="q" />
    %s</form>"""

    def call(self, view=None):
        req = self._cw
        if hasattr(view, "filter_box_context_info"):
            rset = view.filter_box_context_info()[0]
        else:
            rset = self.cw_rset
        # display multilines query as one line
        rql = rset is not None and rset.printable_rql() or req.form.get("rql", "")
        rql = rql.replace("\n", " ")
        rql_suggestion_comp = self._cw.vreg["components"].select_or_none(
            "rql.suggestions", self._cw
        )
        if rql_suggestion_comp is not None:
            # enable autocomplete feature only if the rql
            # suggestions builder is available
            self._cw.add_css("jquery.ui.css")
            self._cw.add_js(("cubicweb.ajax.js", "jquery.ui.js"))
            self._cw.add_onload(
                '$("#rql").autocomplete({source: "%s"});'
                % (req.build_url("json", fname="rql_suggest"))
            )
        hidden = ""
        if req.search_state[0] != "normal":
            hidden = '<input type="hidden" name="__mode" value="%s"/>' % ":".join(
                req.search_state[1]
            )
        self.w(
            self.formdef,
            req.build_url("view"),
            not self.cw_propval("visible") and "hidden" or "",
            rql,
            req._("full text or RQL query"),
            "",
            "",
            hidden,
        )


def registration_callback(vreg):
    components = (
        (BSAuthenticatedUserStatus, basecomponents.AuthenticatedUserStatus),
        (BSRQLInputForm, basecomponents.RQLInputForm),
    )
    vreg.register_all(globals().values(), __name__, [new for (new, old) in components])
    for new, old in components:
        vreg.register_and_replace(new, old)
    vreg.unregister(basecomponents.AnonUserStatusLink)



"""fluid_design_system implementation of ibreadcrumbs

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb.entity import Entity
from cubicweb_web.views import ibreadcrumbs


ibreadcrumbs.BreadCrumbEntityVComponent.css_class = "breadcrumb"


@monkeypatch(ibreadcrumbs.BreadCrumbEntityVComponent)
def open_breadcrumbs(self, w):
    w('<ul id="breadcrumbs" class="%s">' % self.css_class)


@monkeypatch(ibreadcrumbs.BreadCrumbEntityVComponent)
def close_breadcrumbs(self, w):
    w("</ul>")


@monkeypatch(ibreadcrumbs.BreadCrumbEntityVComponent)
def render_breadcrumbs(self, w, contextentity, path):
    root = path.pop(0)
    if isinstance(root, Entity):
        w("<li>")
        w(
            self.link_template
            % (self._cw.build_url(root.__regid__), root.dc_type("plural"))
        )
        w("</li>")
    liclass = ' class="active"' if not path else ""
    w("<li%s>" % liclass)
    self.wpath_part(w, root, contextentity, not path)
    w("</li>")
    for i, parent in enumerate(path):
        last = i == len(path) - 1
        liclass = ' class="active"' if last else ""
        w("<li%s>" % liclass)
        self.wpath_part(w, parent, contextentity, last)
        w("</li>")


@monkeypatch(ibreadcrumbs.BreadCrumbAnyRSetVComponent)
def render(self, w, **kwargs):
    self.open_breadcrumbs(w)
    w('<li class="active">')
    w(self._cw._("search"))
    w("</li>")
    self.close_breadcrumbs(w)



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of navigation components

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

from logilab.common.decorators import monkeypatch

from logilab.mtconverter import xml_escape

from cubicweb_web.view import View
from cubicweb_web.component import NavigationComponent

from cubicweb_web.views.navigation import (
    NextPrevNavigationComponent,
    SortedNavigation,
    PageNavigation,
    PageNavigationSelect,
)

__docformat__ = "restructuredtext en"

NavigationComponent.page_link_templ = '<li><a href="%s" title="%s">%s</a></li>'
NavigationComponent.selected_page_link_templ = (
    '<li class="active"><a href="%s" title="%s">%s</a></li>'
)
NavigationComponent.previous_page_link_templ = NavigationComponent.page_link_templ
NavigationComponent.next_page_link_templ = NavigationComponent.page_link_templ
NavigationComponent.no_previous_page_link = (
    '<li class="disabled"><a href="#">«</a></li>'
)
NavigationComponent.no_next_page_link = '<li class="disabled"><a href="#">»</a></li>'
NavigationComponent.no_content_prev_link = "«"
NavigationComponent.no_content_next_link = "»"


@monkeypatch(NavigationComponent)
def render_link_back_to_pagination(self, w):
    "allow to come back to the paginated view"
    req = self._cw
    params = dict(req.form)
    basepath = req.relative_path(includeparams=False)
    del params["__force_display"]
    url = self.page_url(basepath, params)
    w(
        '<ul class="pagination"><li>'
        '<span><a href="%s">%s</a></span>'
        "</li></ul>"
        % (xml_escape(url), req._("back to pagination (%s results)") % self.page_size)
    )


@monkeypatch(NavigationComponent)
def render_link_display_all(self, w):
    "make a link to see them all"
    req = self._cw
    params = dict(req.form)
    self.clean_params(params)
    basepath = req.relative_path(includeparams=False)
    params["__force_display"] = 1
    params["__fromnavigation"] = 1
    url = self.page_url(basepath, params)
    w(
        '<ul class="pagination pull-right"><li>'
        '<span><a href="%s">%s</a></span>'
        "</li></ul>" % (xml_escape(url), req._("show %s results") % len(self.cw_rset))
    )


@monkeypatch(SortedNavigation)
def write_links(self, basepath, params, blocklist):
    """Return HTML for the whole navigation: `blocklist` is a list of HTML
    snippets for each page, `basepath` and `params` will be necessary to
    build previous/next links.
    """
    self.w('<div class="pagination">')
    self.w("%s&#160;", self.previous_link(basepath, params), escape=False)
    self.w("&#160;%s&#160;", "&#160;| ".join(blocklist), escape=False)
    self.w("&#160;%s", self.next_link(basepath, params), escape=False)
    self.w("</div>")


@monkeypatch(PageNavigation)
def call(self):
    """displays a resultset by page"""
    params = dict(self._cw.form)
    self.clean_params(params)
    basepath = self._cw.relative_path(includeparams=False)
    self.w('<div class="pagination">')
    self.w(self.previous_link(basepath, params))
    self.w(
        "&#160;%s&#160;",
        "&#160;| ".join(self.iter_page_links(basepath, params)),
        escape=False,
    )
    self.w("&#160;&#160;%s", self.next_link(basepath, params), escape=False)
    self.w("</div>")


NextPrevNavigationComponent.prev_icon = "&#8592;"
NextPrevNavigationComponent.next_icon = "&#8594;"


@monkeypatch(NextPrevNavigationComponent)
# Should be better done, but this works
def render_body(self, w):
    w('<div class="prevnext row">')
    w('<ul class="pager">')
    self.prevnext(w)
    w("</ul>")
    w("</div>")
    # w(u'<div class="clear"></div>')


@monkeypatch(NextPrevNavigationComponent)
def prevnext_div(self, w, type, cssclass, url, title, content):
    csscls = {"prev": "previous", "next": "next"}
    w('<li class="%s">' % csscls[type])
    w('<a href="%s" title="%s">%s</a>' % (xml_escape(url), xml_escape(title), content))
    w("</li>")
    self._cw.html_headers.add_raw(
        '<link rel="%s" href="%s" />' % (type, xml_escape(url))
    )


def do_paginate(view, rset=None, w=None, show_all_option=True, page_size=None):
    """write pages index in w stream (default to view.w) and then limit the
    result set (default to view.rset) to the currently displayed page if we're
    not explicitly told to display everything (by setting __force_display in
    req.form)
    """
    req = view._cw
    if rset is None:
        rset = view.cw_rset
    if w is None:
        w = view.w
    nav = req.vreg["components"].select_or_none(
        "navigation", req, rset=rset, page_size=page_size, view=view
    )
    if nav:
        if w is None:
            w = view.w
        if req.form.get("__force_display"):
            # allow to come back to the paginated view
            params = dict(req.form)
            basepath = req.relative_path(includeparams=False)
            del params["__force_display"]
            url = nav.page_url(basepath, params)
            w(
                '<span><a href="%s">%s</a></span>\n'
                % (
                    xml_escape(url),
                    req._("back to pagination (%s results)") % nav.page_size,
                )
            )
        else:
            # get boundaries before component rendering
            start, stop = nav.page_boundaries()
            nav.render(w=w)
            params = dict(req.form)
            nav.clean_params(params)
            # make a link to see them all
            if show_all_option:
                basepath = req.relative_path(includeparams=False)
                params["__force_display"] = 1
                params["__fromnavigation"] = 1
                url = nav.page_url(basepath, params)
                w(
                    '<span><a href="%s">%s</a></span>\n'
                    % (xml_escape(url), req._("show %s results") % len(rset))
                )
            rset.limit(offset=start, limit=stop - start, inplace=True)  # noqa


def paginate(view, show_all_option=True, w=None, page_size=None, rset=None):
    """paginate results if the view is paginable"""
    if view.paginable:
        do_paginate(view, rset, w, show_all_option, page_size)


# monkey patch base View class to add a .paginate([...])
# method to be called to write pages index in the view and then limit the result
# set to the current page


View.handle_pagination = False


@monkeypatch(PageNavigationSelect)  # noqa: F811
def call(self):
    params = dict(self._cw.form)
    self.clean_params(params)
    basepath = self._cw.relative_path(includeparams=False)
    self.w('<ul class="pagination">')
    self.w(self.previous_link(basepath, params))
    start, stop = self.page_boundaries()
    self.w(
        '<li><span><a class="dropdown-toggle" data-toggle="dropdown" href="#">'
        '%s<span class="caret"></span></a>',
        self.index_display(start, stop - 1),
    )
    self.w('<ul class="dropdown-menu" role="menu">')
    for option in self.iter_page_links(basepath, params):
        self.w(option)
    self.w("</ul></span></li>")
    self.w("&#160;%s", self.next_link(basepath, params), escape=False)
    self.w("</ul>")


PageNavigationSelect.page_link_templ = '<li><a href="%s" title="%s">%s</a></li>'
PageNavigationSelect.selected_page_link_templ = (
    '<li class="disabled"><a href="%s" title="%s">%s</a></li>'
)



# copyright 2013-2021 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <https://www.gnu.org/licenses/>.
"""fluid_design_system implementation of forms"""

__docformat__ = "restructuredtext en"


from cubicweb_web.views.forms import FieldsForm
from cubicweb_web.views.autoform import AutomaticEntityForm

# Forms
FieldsForm.needs_js += ("cubes.bootstrap.edition.js",)
FieldsForm.needs_css = ()
FieldsForm.cssclass = AutomaticEntityForm.cssclass = "form-horizontal"



# copyright 2013-2021 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <https://www.gnu.org/licenses/>.
"""fluid_design_system implementation of formrenderers"""

__docformat__ = "restructuredtext en"

from warnings import warn

from logilab.common.decorators import monkeypatch

from logilab.mtconverter import xml_escape

from cubicweb import tags
from cubicweb.utils import support_args
from cubicweb_web.views import formrenderers


@monkeypatch(formrenderers.EntityCompositeFormRenderer)
def render_fields(self, w, form, values):
    if form.parent_form is None:
        # We should probably take those CSS classes to uiprops.py
        w('<table class="table table-striped table-bordered table-condensed">')
        # get fields from the first subform with something to display (we
        # may have subforms with nothing editable that will simply be
        # skipped later)
        for subform in form.forms:
            subfields = [field for field in subform.fields if field.is_visible()]
            if subfields:
                break
        if subfields:
            # main form, display table headers HTML5
            w("<thead>")
            w("<tr>")
            w(
                "<th>%s</th>"
                % tags.input(
                    type="checkbox",
                    title=self._cw._("toggle check boxes"),
                    onclick="setCheckboxesState('eid', null, this.checked)",
                )
            )
            for field in subfields:
                w("<th>%s</th>" % formrenderers.field_label(form, field))
            w("</tr>")
            w("</thead>")
    super(formrenderers.EntityCompositeFormRenderer, self).render_fields(
        w, form, values
    )
    if form.parent_form is None:
        w("</table>")
        if self._main_display_fields:
            super(formrenderers.EntityCompositeFormRenderer, self)._render_fields(
                self._main_display_fields, w, form
            )


formrenderers.FormRenderer.button_bar_class = "form-group"


class ModalFormRenderer(formrenderers.FormRenderer):
    __regid__ = "modal-form-renderer"
    button_bar_class = "modal-footer"

    def open_form(self, form, values, **attrs):
        showmessage = values.get("showmessage")
        showonload = values.get("showonload", False)
        if showonload:
            # show the `modal` dialog : initialized in
            # basecomponents.CookieLoginComponent by showonload=True
            self._cw.add_onload("$('#%s').modal('show')" % values["modal_id"])
            data_backdrop, data_keyboard = "static", "false"
        else:
            data_backdrop, data_keyboard = "true", "true"

        html = [
            '<div class="%(class)s" id="%(id)s" tabindex="-1" role="dialog" '
            'data-backdrop="%(backdrop)s" data-keyboard="%(keyboard)s">\n'
            '<div class="modal-dialog">'
            '<div class="modal-content">'
            % {
                "id": values["modal_id"],
                "class": "modal fade in" if showmessage else "modal",
                "backdrop": data_backdrop,
                "keyboard": data_keyboard,
            }
        ]
        html.append(super(ModalFormRenderer, self).open_form(form, values, **attrs))
        html.append('<div class="modal-header">')
        if not showonload:
            # add cancel button when modal triggered by CookieLoginComponent
            html.append(
                '<button type="button" class="close" data-dismiss="modal" '
                'aria-hidden="true">&#215;</button>'
            )
        html.append(
            '<div class="modal-title">%s</div>\n</div>\n'
            % xml_escape(values.get("title", ""))
        )
        return "\n".join(html)

    def render_content(self, w, form, values):
        self._cw.add_onload(
            "$('.close').click(function () {$(this).parent().removeClass('in');});"
        )
        w('<div class="modal-body">')
        if values.get("showmessage"):
            message = self._cw.message
            if message:
                w(
                    '<div class="alert alert-danger in">%s'
                    '<button class="close" data-dismiss="alert">x</button></div>',
                    xml_escape(message),
                )
        if self.display_progress_div:
            w('<div id="progress">%s</div>' % self._cw._("validating..."))
        w("\n<fieldset>\n")
        self.render_fields(w, form, values)
        w("\n</fieldset>\n")
        w("</div>")
        self.render_buttons(w, form)

    def close_form(self, form, values):
        html = [super(ModalFormRenderer, self).close_form(form, values)]
        html.append("</div></div></div>")  # close modal-content, modal-dialog, ...
        return "\n".join(html)


@monkeypatch(formrenderers.FormRenderer)
def render_help(self, form, field):
    """display help in the form"""
    help = []
    descr = field.help
    if callable(descr):
        if support_args(descr, "form", "field"):
            descr = descr(form, field)
        else:
            warn(
                "[3.10] field's help callback must now take form "
                "and field as argument (%s)" % field,
                DeprecationWarning,
            )
            descr = descr(form)
    if descr:
        help.append('<p class="help-block">%s</p>' % self._cw._(descr))
    example = field.example_format(self._cw)
    if example:
        help.append(
            '<p class="form-control-static">(%s: %s)</p>'
            % (self._cw._("sample format"), example)
        )
    return "&#160;".join(help)


@monkeypatch(formrenderers.FormRenderer)
def error_message(self, form):
    """return formatted error message

    This method should be called once inlined field errors has been consumed
    """
    req = self._cw
    errex = form.form_valerror
    # get extra errors
    if errex is not None:
        errormsg = req._("please correct the following errors:")
        errors = form.remaining_errors()
        if errors:
            if len(errors) > 1:
                templstr = "<li>%s</li>"
            else:
                templstr = "&#160;%s"
            for field, err in errors:
                if field is None:
                    errormsg += templstr % err
                else:
                    errormsg += templstr % "%s: %s" % (req._(field), err)
            if len(errors) > 1:
                errormsg = "<ul>%s</ul>" % errormsg
        return '<div class="alert alert-danger">%s</div>' % errormsg
    return ""


@monkeypatch(formrenderers.FormRenderer)
def _render_fields(self, fields, w, form):
    """render form fields"""
    byfieldset = {}
    for field in fields:
        byfieldset.setdefault(field.fieldset, []).append(field)
    if form.fieldsets_in_order:
        fieldsets = form.fieldsets_in_order
    else:
        fieldsets = byfieldset
    for fieldset in list(fieldsets):
        try:
            fields = byfieldset.pop(fieldset)
        except KeyError:
            self.warning("no such fieldset: %s (%s)", fieldset, form)
            continue
        w("<fieldset>\n")
        if fieldset:
            w("<legend>%s</legend>" % self._cw.__(fieldset))
        for field in fields:
            error = form.field_error(field)
            control = not hasattr(field, "control_field") or field.control_field
            w(
                '<div id="%s-%s_row" class="form-group %s">'
                % (field.name, field.role, "error" if error else "")
            )
            if self.display_label and field.label is not None:
                w("%s" % self.render_label(form, field))
            # Use full width for inlined forms, 9/12 for normal fields
            base_css = "col-md-12" if hasattr(field, "view") else "col-md-9"
            if not control:
                # katia : 'control' class no longer exists in fluid_design_system 3.0.0
                # but is used here because of
                # 'form-horizontal' fluid_design_system 2.0.4 css
                # backport
                base_css += " nomargin"
            w('<div class="%s">' % base_css)
            w(field.render(form, self))
            if error:
                self.render_error(w, error)
            if self.display_help:
                w(self.render_help(form, field))
            w("</div>")
            w("</div>")
        w("</fieldset>")
    if byfieldset:
        self.warning("unused fieldsets: %s", ", ".join(byfieldset))


@monkeypatch(formrenderers.FormRenderer)
def render_label(self, form, field):
    if field.label is None:
        return ""
    label = formrenderers.field_label(form, field)
    attrs = {"for": field.dom_id(form)}
    attrs["class"] = "col-md-3 control-label"
    if field.required:
        attrs["class"] += " required"
    return tags.label(label, **attrs)


@monkeypatch(formrenderers.FormRenderer)
def render_buttons(self, w, form):
    """render form's buttons"""
    if not form.form_buttons:
        return
    w('<div class="%s">' % self.button_bar_class)
    for button in form.form_buttons:
        w(button.render(form))
    w("</div>")


@monkeypatch(formrenderers.EntityFormRenderer)
def open_form(self, form, values):
    attrs_fs_label = ""
    if self.main_form_title:
        attrs_fs_label = "<h3>%s</h3>" % self._cw._(self.main_form_title)
    open_form = attrs_fs_label + super(
        formrenderers.EntityFormRenderer, self
    ).open_form(form, values)
    return open_form


@monkeypatch(formrenderers.EntityFormRenderer)
def close_form(self, form, values):
    # needed to remove the '</div>' from the original method
    return super(formrenderers.EntityFormRenderer, self).close_form(form, values)


@monkeypatch(formrenderers.EntityFormRenderer)  # noqa: F811
def render_buttons(self, w, form):
    # needed to use our own monkeypatched FormRenderer.render_buttons() which
    # doesn't need to add a special case when there are 3 buttons to render
    super(formrenderers.EntityFormRenderer, self).render_buttons(w, form)


@monkeypatch(formrenderers.EntityInlinedFormRenderer)  # noqa: F811
def open_form(self, w, form, values):
    try:
        w('<div id="div-%(divid)s" onclick="%(divonclick)s">' % values)
    except KeyError:
        w('<div id="div-%(divid)s">' % values)
    else:
        w(
            '<div id="notice-%s" class="notice text-info">'
            '<span class="glyphicon glyphicon-info-sign"></span> %s</div>'
            % (values["divid"], self._cw._("click on the box to cancel the deletion"))
        )
    w('<div class="iformBody">')


@monkeypatch(formrenderers.EntityInlinedFormRenderer)  # noqa: F811
def close_form(self, w, form, values):
    w("</div></div>")


@monkeypatch(formrenderers.EntityInlinedFormRenderer)  # noqa: F811
def render_fields(self, w, form, values):
    w('<fieldset id="fs-%(divid)s">' % values)
    fields = self._render_hidden_fields(w, form)
    w("</fieldset>")
    if fields:
        self._render_fields(fields, w, form)
    self.render_child_forms(w, form, values)



# pylint: disable=W0622
"""cubicweb-fluid-design-system application packaging information"""

modname = "fluid_design_system"
distname = "cubicweb-" + modname.replace('_', '-')

numversion = (1, 10, 4)
version = ".".join(str(num) for num in numversion)

license = "LGPL"
author = "LOGILAB S.A. (Paris, FRANCE)"
author_email = "contact@logilab.fr"
description = "CubicWeb's cube used to manage the ENGIE's Fluid Design System"
web = "https://github.tools.digital.engie.com/GBSEngieDigitalNemoOpti/cubicweb-fds"

__depends__ = {
    "cubicweb": ">= 4.5.2, < 5.0.0",
    "cubicweb-web": ">= 1.3.1, < 2.0",
    "webtest": "",
}
__recommends__ = {}

classifiers = [
    "Environment :: Web Environment",
    "Framework :: CubicWeb",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3 :: Only",
    "Programming Language :: JavaScript",
]



# copyright 2013-2021 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <https://www.gnu.org/licenses/>.
"""this is where you could register procedures for instance"""
__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb_web import formwidgets, stdmsgs

from cubicweb_fluid_design_system import monkeypatch_default_value

# put there monkeypatches of things that aren't reloaded automatically

# monkeypatches #############################################################

# keep the original `formwidgets.FieldWidget.attributes`
orig_attributes = formwidgets.FieldWidget.attributes

formwidgets.FieldWidget.css_class = "form-control"

# Keep the `validateButton` as it is used in js (e.g `unfreezeFormButtons`)
formwidgets.ButtonInput.css_class = "btn btn-default"
formwidgets.Button.css_class = "btn btn-default validateButton"
formwidgets.SubmitButton.css_class = "btn btn-primary validateButton"


@monkeypatch(formwidgets.FieldWidget)
def attributes(self, form, field):
    """add the bootstrap `form-control` class to formwidgets.FieldWidget.attributes method"""
    attrs = orig_attributes(self, form, field)
    if self.css_class:
        attrs["class"] = " ".join((attrs.get("class", ""), self.css_class)).strip()
    return attrs


# in bootstrap 3.0.0 CheckBox, RadioBox and FileInput must not have
# form-control class


formwidgets.FileInput.attributes = orig_attributes
formwidgets.CheckBox.attributes = orig_attributes


# Buttons
stdmsgs.BUTTON_OK = (stdmsgs.BUTTON_OK[0], "glyphicon glyphicon-ok")
stdmsgs.BUTTON_APPLY = (stdmsgs.BUTTON_APPLY[0], "glyphicon glyphicon-cog")
stdmsgs.BUTTON_CANCEL = (stdmsgs.BUTTON_CANCEL[0], "glyphicon glyphicon-remove")
stdmsgs.BUTTON_DELETE = (stdmsgs.BUTTON_DELETE[0], "glyphicon glyphicon-trash")


# Override default form buttons to 'btn-class' and use bootstrap glyphicons.


monkeypatch_default_value(
    formwidgets.Button.__init__,
    "label",
    (stdmsgs.BUTTON_OK[0], "glyphicon glyphicon-ok"),
)


# options #############################################################

options = (
    (
        "fluid_design_system_compatibility",
        {
            "type": "yn",
            "default": False,
            "help": "use compat css rules (enable if your application uses views "
            "that have not been ported to bootstrap)",
            "group": "fluid_design_system",
            "level": 2,
        },
    ),
)



"""
:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

from logilab.common.decorators import monkeypatch

from cubicweb_web import view as cwview
from cubicweb.schema import display_name
from cubicweb_web.views.basetemplates import HTMLHeader
from cubicweb_web.views.boxes import ContextualBoxLayout, contextual

# do not wrap cell_calls with <div class="section">

cwview.View.add_div_section = False


@monkeypatch(cwview.View)
def field(self, label, value, row=True, show_label=True, w=None, tr=True, table=False):
    """read-only field"""
    if w is None:
        w = self.w
    w("<tr>")
    if show_label and label:
        if tr:
            label = display_name(self._cw, label)
        w("<th>%s</th>" % label)
    if not (show_label and label):
        w('<td colspan="2">%s</td>' % value)
    else:
        w("<td>%s</td>" % value)
    w("</tr>")


class BSHTMLHeader(HTMLHeader):
    def stylesheets(self):
        super(BSHTMLHeader, self).stylesheets()
        # add cw compatibility stylesheets
        if self._cw.vreg.config["fluid_design_system_compatibility"]:
            for css in self._cw.uiprops["CW_COMPAT_STYLESHEETS"]:
                self._cw.add_css(css, localfile=False)


class SimpleBoxContextFreeBoxLayout(ContextualBoxLayout):
    __select__ = ~contextual()
    cssclass = "contextFreeBox"
    __regid__ = "simple-layout"

    def render(self, w):
        if self.init_rendering():
            view = self.cw_extra_kwargs["view"]
            view.render_body(w)


def registration_callback(vreg):
    components = ((BSHTMLHeader, HTMLHeader),)
    vreg.register_all(globals().values(), __name__, [new for (new, old) in components])
    for new, old in components:
        vreg.register_and_replace(new, old)



"""fluid_design_system implementation of facets

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from cubicweb_web.views import facets

facets.FilterBox.bk_linkbox_template = '<div class="facet-action">%s</div>'



# -*- coding: utf-8 -*-
"""fluid_design_system html helpers

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"


class DropDownBox:
    ul_template = """
    <div class="nj-avatar nj-avatar--sm">
        <div class="nj-avatar__picture"
             data-toggle="collapse"
             href="#dropdownMenu"
             aria-expanded="false" aria-controls="dropdownMenu" role="button">
            <div class="nj-avatar__initials">%(title)s</div>
        </div>
        <div class="nj-collapse" id="dropdownMenu">
            <ul class="dropdown-menu nj-list-group nj-list-group--sm nj-list-group--no-border"
                role="menu">%(actions)s</ul>
        </div>
    </div>"""

    li_template = "<li>%(link)s<li>"

    def __init__(self, title, actions, klass=""):
        self.title = title
        self.actions = actions
        self.klass = klass

    def render(self, w):
        if not len(self.actions):
            return ""
        w(
            self.ul_template
            % {
                "title": self.title,
                "actions": "".join(self.render_items()),
            }
        )

    def render_items(self):
        for item in self.actions:
            yield self.li_template % {"link": self._item_value(item)}

    def _item_value(self, item):
        return item



"""fluid_design_system implementation of base debug views

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from time import strftime, localtime

from logilab.common.decorators import monkeypatch

from logilab.mtconverter import xml_escape

from cubicweb import BadConnectionId
from cubicweb_web.views import debug


def dict_to_html(w, dict):
    # XHTML doesn't allow emtpy <ul> nodes
    if dict:
        w("<dl>")
        for key in sorted(dict):
            w(
                "<dt>%s</dt><dd>%s</dd>"
                % (xml_escape(str(key)), xml_escape(repr(dict[key])))
            )
        w("</dl>")


@monkeypatch(debug.ProcessInformationView)
def call(self, **kwargs):
    req = self._cw
    dtformat = req.property_value("ui.datetime-format")
    _ = req._
    repo = req.cnx.repo
    # generic instance information
    self.w("<h2>%s</h2>", _("Instance"))
    self.w('<table class="table table-striped table-condensed">')
    for key, value in (
        (_("config type"), self._cw.vreg.config.name),
        (_("config mode"), self._cw.vreg.config.mode),
        (_("instance home"), self._cw.vreg.config.apphome),
    ):
        self.w("<tr><th>%s</th><td>%s</td></tr>", key, value)
    self.w("</table>")
    vcconf = repo.get_versions()
    self.w("<h3>%s</h3>", _("versions configuration"))
    self.w('<table class="table table-striped table-condensed">')
    self.w(
        "<tr><th>%s</th><td>%s</td></tr>",
        "CubicWeb",
        vcconf.get("cubicweb", _("no version information")),
    )
    for cube in sorted(self._cw.vreg.config.cubes()):
        cubeversion = vcconf.get(cube, _("no version information"))
        self.w("<tr><th>%s</th><td>%s</td></tr>", cube, cubeversion)
    self.w("</table>")
    # repository information
    self.w("<h2>%s</h2>", _("Repository"))
    self.w("<h3>%s</h3>", _("resources usage"))
    self.w('<table class="table table-striped table-condensed">')
    stats = self._cw.call_service("repo_stats")
    for element in sorted(stats):
        self.w(
            "<tr><th>%s</th><td>%s %s</td></tr>",
            element,
            str(stats[element]),
            element.endswith("percent") and "%" or "",
        )
    self.w("</table>")
    # web server information
    self.w("<h2>%s</h2>", _("Web server"))
    self.w('<table class="table table-striped table-condensed">')
    self.w("<tr><th>%s</th><td>%s</td></tr>", _("base url"), req.base_url())
    self.w("<tr><th>%s</th><td>%s</td></tr>", _("data directory url"), req.datadir_url)
    self.w("</table>")
    from cubicweb_web.application import SESSION_MANAGER

    if SESSION_MANAGER is not None and req.user.is_in_group("managers"):
        sessions = SESSION_MANAGER.current_sessions()
        self.w("<h3>%s</h3>", _("opened web sessions"))
        if sessions:
            self.w("<ul>")
            for session in sessions:
                if hasattr(session, "cnx"):
                    # cubicweb < 3.19
                    if not session.cnx:
                        self.w("<li>%s (NO CNX)</li>", session.sessionid)
                        continue
                    try:
                        last_usage_time = session.cnx.check()
                    except BadConnectionId:
                        self.w("<li>%s (INVALID)</li>", session.sessionid)
                        continue
                else:
                    # cubicweb >= 3.19
                    last_usage_time = session.mtime
                self.w(
                    "<li>%s (%s: %s)<br/>",
                    session.sessionid,
                    _("last usage"),
                    strftime(dtformat, localtime(last_usage_time)),
                )
                dict_to_html(self.w, session.data)
                self.w("</li>")
            self.w("</ul>")
        else:
            self.w("<p>%s</p>", _("no web sessions found"))


@monkeypatch(debug.RegistryView)  # noqa: F811
def call(self, **kwargs):
    self.w("<h2>%s</h2>", self._cw._("Registry's content"))
    keys = sorted(self._cw.vreg)
    url = xml_escape(self._cw.url())
    self.w(
        "<p>%s</p>\n",
        " - ".join(
            '<a href="%s#%s">%s</a>'
            % (xml_escape(url), xml_escape(key), xml_escape(key))
            for key in keys
        ),
        escape=False,
    )
    for key in keys:
        if key in ("boxes", "contentnavigation"):  # those are bw compat registries
            continue
        self.w('<h3 id="%s">%s</h3>', key, key)
        if self._cw.vreg[key]:
            values = sorted(self._cw.vreg[key].items())
            self.wview(
                "pyvaltable",
                pyvalue=[(key, xml_escape(repr(val))) for key, val in values],
            )
        else:
            self.w("<p>Empty</p>\n")



"""fluid_design_system implementation of baseviews

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb_web.views import baseviews

baseviews.ListView.listtag = "ul"


@monkeypatch(baseviews.ListView)
def call(self, klass="list-striped", title=None, subvid=None, listid=None, **kwargs):
    """display a list of entities by calling their <item_vid> view

    :param listid: the DOM id to use for the root element
    """
    if subvid is None and "subvid" in self._cw.form:
        subvid = self._cw.form.pop("subvid")  # consume it
    if listid:
        listid = ' id="%s"' % listid
    else:
        listid = ""
    if title:
        self.w('<div%s class="%s"><h4>%s</h4>\n', listid, klass or "", title)
        self.w('<%s class="%s">\n', self.listtag, klass or "")
    else:
        self.w('<%s%s class="%s">\n', self.listtag, listid, klass or "")
    for i in range(self.cw_rset.rowcount):
        self.cell_call(row=i, col=0, vid=subvid, klass=klass, **kwargs)
    self.w("</%s>\n", self.listtag)
    if title:
        self.w("</div>\n")


@monkeypatch(baseviews.ListView)
def cell_call(self, row, col=0, vid=None, klass=None, **kwargs):
    self.w('<li class="%s">', "odd" if row % 2 else "even")
    self.wview(self.item_vid, self.cw_rset, row=row, col=col, vid=vid, **kwargs)
    self.w("</li>\n")


@monkeypatch(baseviews.SameETypeListView)  # noqa: F811
def call(self, klass="list-striped", **kwargs):
    showtitle = kwargs.pop("showtitle", "vtitle" not in self._cw.form)
    if showtitle:
        self.w("<h1>%s</h1>", self.title)
    klass = "" if klass is None else ' class="%s"' % klass
    self.w("<ul%s>\n", klass)
    for i in range(len(self.cw_rset)):
        self.w('<li class="%s">', "odd" if i % 2 else "even")
        self._cw.view(self.item_vid, self.cw_rset, row=i, col=0, w=self.w)
        self.w("</li>")
    self.w("</ul>\n")


@monkeypatch(baseviews.MetaDataView)  # noqa: F811
def cell_call(self, row, col):
    _ = self._cw._
    entity = self.cw_rset.get_entity(row, col)
    self.w('<p class="text-right"><small>')
    if self.show_eid:
        self.w("%s #%s - ", entity.dc_type(), entity.eid)
    if entity.modification_date != entity.creation_date:
        self.w("<span>%s</span> ", _("latest update on"))
        self.w(
            '<span class="value">%s</span>, ',
            self._cw.format_date(entity.modification_date),
        )
    # entities from external source may not have a creation date (eg ldap)
    if entity.creation_date:
        self.w("<span>%s</span> ", _("created on"))
        self.w(
            '<span class="value">%s</span>', self._cw.format_date(entity.creation_date)
        )
    if entity.creator:
        if entity.creation_date:
            self.w(" <span>%s</span> ", _("by"))
        else:
            self.w(" <span>%s</span> ", _("created_by"))
        self.w('<span class="value">%s</span>', entity.creator.name())
    if entity.cw_source:
        source = entity.cw_source[0]
        if source.name != "system":
            self.w(" (<span>%s</span>", _("cw_source"))
            self.w(' <span class="value">%s</span>)', source.name)
    self.w("</small></p>")
